@@ -7,7 +7,7 @@ import functools
 
 import matplotlib
 
-from skrt.core import Archive
+from skrt.core import Archive, Data
 import skrt.image
 import skrt.structures
 
@@ -17,9 +17,9 @@ class ImageOverlay(skrt.image.Image):
     functionality includes the ability to plot overlaid on its associated 
     Image."""
 
-    def __init__(self, path, load=True, image=None, *args, **kwargs):
+    def __init__(self, path="", load=True, image=None, *args, **kwargs):
 
-        skrt.image.Image.__init__(self, path, *args, **kwargs)
+        skrt.image.Image.__init__(self, path, load, *args, **kwargs)
         self.set_image(image)
 
         # Default dose plotting settings
@@ -166,10 +166,18 @@ class Dose(ImageOverlay):
 
         ImageOverlay.__init__(self, *args, **kwargs)
 
+        # Ensure linking between image and dose
+        self.set_image(self.image)
+
         # Plot settings specific to dose map
         self._default_cmap = "jet"
         self._default_colorbar_label = "Dose (Gy)"
 
+        # Delete spurious attributes inherited from Image class
+        for attribute in ['doses', 'plans']:
+            if hasattr(self, attribute):
+                delattr(self, attribute)
+
     def load(self, *args, **kwargs):
         """Load self and set default maximum plotting intensity from max of
         data array if not yet set."""
@@ -184,6 +192,20 @@ class Dose(ImageOverlay):
         if image is not None:
             image.add_dose(self)
 
+    def set_plan(self, plan):
+        """Set associated plan, initialising it if needed."""
+
+        # Convert to Plan object if needed
+        if plan and not isinstance(plan, Plan):
+            plan = Plan(plan)
+
+        # Assign plan to self
+        self.plan = plan
+
+        # Assign self to the plan
+        if self.plan is not None:
+            self.plan.add_dose(self)
+
     def view(self, **kwargs):
 
         return ImageOverlay.view(self, kwarg_name="dose", **kwargs)
@@ -220,7 +242,8 @@ class Dose(ImageOverlay):
 
     def plot_DVH(self, rois=[], bins=50, dose_min=0, dose_max=None,
             figsize=(8, 4), lw=2, n_colour=10, cmap='turbo', grid=True,
-            fname=None):
+            fname=None, legend_bbox_to_anchor=(1.01, 0.5),
+            legend_loc='center left'):
         '''
         Plot dose-volume histogram for specified ROI(s).
 
@@ -308,8 +331,8 @@ class Dose(ImageOverlay):
         ax.set_xlabel('Dose (Gy)')
         ax.set_ylabel('Volume fraction')
         ax.grid(grid)
-        ax.legend(handles=lines, labels=labels, loc='center left',
-                bbox_to_anchor=(1.01, 0.5))
+        ax.legend(handles=lines, labels=labels,
+                bbox_to_anchor=legend_bbox_to_anchor, loc=legend_loc)
         matplotlib.pyplot.tight_layout()
 
         # Show figure or save to file.
@@ -331,15 +354,24 @@ class Plan(Archive):
 
         self.loaded = False
         self.path = None
+        self.name = None
         self.approval_status = None
         self.n_fraction_group = None
         self.n_beam_seq = None
         self.n_fraction = None
+        self.organs_at_risk = None
         self.target_dose = None
+        self.targets = None
         self.image = None
         self.structure_set = None
+        self.doses = []
+        self.objectives = Data()
+        for constraint_attribute in Constraint.get_weight_and_objectives():
+            setattr(self.objectives, constraint_attribute, None)
+
         Archive.__init__(self, path)
 
+        self.constraints_loaded = False
         if load:
             self.load()
 
@@ -353,6 +385,8 @@ class Plan(Archive):
 
         self.dicom_dataset = pydicom.dcmread(self.path, force=True)
 
+        self.name = getattr(self.dicom_dataset, 'RTPlanName', None)
+
         try:
             self.approval_status = self.dicom_dataset.ApprovalStatus
         except AttributeError:
@@ -360,7 +394,7 @@ class Plan(Archive):
 
         try:
             self.n_fraction_group = len(
-                    self.dicom.dataset.FractionGroupSequence)
+                    self.dicom_dataset.FractionGroupSequence)
         except AttributeError:
             self.n_fraction_group = None
 
@@ -370,7 +404,6 @@ class Plan(Archive):
             self.n_beam_seq = None
 
         self.n_fraction = None
-        self.target_dose = None
         if self.n_fraction_group is not None:
             self.n_fraction = 0
             for fraction in self.dicom_dataset.FractionGroupSequence:
@@ -382,6 +415,49 @@ class Plan(Archive):
                         self.target_dose += dose.TargetPrescriptionDose
 
         self.loaded = True
+        self.load_constraints()
+
+    def load_constraints(self, force=False):
+        '''
+        Load dose constraints from plan.
+        '''
+
+        if ((self.constraints_loaded and not force) or not self.structure_set):
+            return
+
+        rois = {}
+        for roi in self.structure_set.get_rois():
+            rois[roi.number] = roi
+
+        self.organs_at_risk = []
+        self.targets = []
+        
+        ds = self.get_dicom_dataset()
+
+        for item in ds.DoseReferenceSequence:
+            roi = rois.get(item.ReferencedROINumber, None)
+            if roi is None:
+                continue
+            roi.roi_type = item.DoseReferenceType
+            roi.constraint = Constraint()
+            roi.constraint.weight = float(item.ConstraintWeight)
+            if 'ORGAN_AT_RISK' == item.DoseReferenceType:
+                roi.constraint.maximum_dose = float(item.OrganAtRiskMaximumDose)
+                roi.constraint.full_volume_dose = float(
+                        item.OrganAtRiskFullVolumeDose)
+                roi.constraint.overdose_volume_fraction = float(
+                        item.OrganAtRiskOverdoseVolumeFraction)
+                self.organs_at_risk.append(roi)
+            elif 'TARGET' == item.DoseReferenceType:
+                roi.constraint.minimum_dose = float(item.TargetMinimumDose)
+                roi.constraint.prescription_dose = float(
+                        item.TargetPrescriptionDose)
+                roi.constraint.maximum_dose = float(item.TargetMaximumDose)
+                roi.constraint.underdose_volume_fraction = float(
+                        item.TargetUnderdoseVolumeFraction)
+                self.targets.append(roi)
+
+        self.constraints_loaded = True
 
     def get_dicom_dataset(self):
         '''
@@ -391,6 +467,109 @@ class Plan(Archive):
         self.load()
         return self.dicom_dataset
 
+    def get_targets(self):
+        '''
+        Return list of ROIs identified in plan as targets.
+        '''
+        self.load()
+        return self.targets
+
+    def get_organs_at_risk(self):
+        '''
+        Return list of ROIs identified in plan as organs at risk.
+        '''
+        self.load()
+        return self.organs_at_risk
+
+    def get_dose_objective(self, objective='maximum_dose', idx_dose=0,
+            dose=None):
+        '''
+        Obtain Dose object representing weight or objective of dose constraint.
+
+        **Parameters:**
+        objective : str, default='maximum_dose'
+            Identifier of objective for which information is to be obtained.
+            For a list of objectives, see skrt.dose.Constraint class.
+
+        idx_dose : int, default=0
+            Index of dose object in self.doses from which dose data are
+            to be taken.
+
+        dose : skrt.dose.Dose, default=None
+            Dose object from which dose data are to be taken.  If specified,
+            idx_dose is ignored.
+        '''
+
+        # Check that specified objective is known.
+        if objective not in Constraint.get_weight_and_objectives():
+            print(f'Unknown dose objective: \'{objective}\'')
+            print(f'Known objectives: {Constraint.get_weight_and_objectives()}')
+            return None
+
+
+        # Return pre-existing result if available.
+        dose_objective = getattr(self.objectives, objective)
+        if dose_objective:
+            return dose_objective
+
+        # Check that data needed for defining dose objective are available.
+        rois = self.get_targets()
+        rois.extend(self.get_organs_at_risk())
+        if not rois or not (self.doses or dose):
+            return dose_objective
+
+        # Initialise Dose object for objective data.
+        if dose:
+            dose_objective = Dose(dose)
+        else:
+            dose_objective = Dose(self.doses[idx_dose])
+
+        dose_objective.load()
+        dose_objective.data = np.zeros(dose_objective.data.shape)
+        
+        # Obtain objective information for each ROI.
+        for roi in rois:
+            if roi.constraint:
+                roi_clone = skrt.structures.ROI(roi)
+                roi_clone.set_image(dose_objective)
+                mask = roi_clone.get_mask()
+                dose_objective.data[mask > 0] = (
+                        getattr(roi_clone.constraint, objective))
+
+        setattr(self.objectives, objective, dose_objective)
+
+        return dose_objective
+
+    def get_approval_status(self):
+        '''Return plan approval status.'''
+        self.load()
+        return self.approval_status
+
+    def get_n_beam_seq(self):
+        '''Return number of beam sequences for this plan.'''
+        self.load()
+        return self.n_beam_seq
+
+    def get_n_fraction(self):
+        '''Return number of fractions for this plan.'''
+        self.load()
+        return self.n_fraction
+
+    def get_n_fraction_group(self):
+        '''Return number of fraction groups for this plan.'''
+        self.load()
+        return self.n_fraction_group
+
+    def get_name(self):
+        '''Return plan name.'''
+        self.load()
+        return self.name
+
+    def get_target_dose(self):
+        '''Return dose to target (tumour) for this plan.'''
+        self.load()
+        return self.name
+
     def set_image(self, image):
         """Set associated image, initialising it if needed."""
 
@@ -419,5 +598,68 @@ class Plan(Archive):
         # Assign self to the structure set and its rois
         if self.structure_set is not None:
             self.structure_set.add_plan(self)
-            for roi in self.structure_set.get_rois():
-                roi.add_plan(self)
+
+    def add_dose(self, dose):
+        """Add a Dose object to be associated with this plan. This does not
+        affect the plan associated with the Dose object.
+
+        **Parameters:**
+
+        dose : skrt.dose.Dose
+            A Dose object to assign to this plan.
+        """
+
+        self.doses.append(dose)
+        self.doses.sort()
+
+    def clear_doses(self):
+        """Clear all dose maps associated with this plan."""
+
+        self.doses = []
+
+
+class Constraint(Data):
+    '''
+    Container for data relating to a dose constraint.
+    '''
+
+    @classmethod
+    def get_weight_and_objectives(cls):
+        '''
+        Return weight and objectives that may be associated with a constraint.
+        '''
+        return ['weight', 'minimum_dose', 'maximum_dose', 'full_volume_dose',
+                'prescription_dose', 'underdose_volume_fraction',
+                'overdose_volume_fraction']
+
+    def __init__(self, opts={}, **kwargs):
+        """
+        Constructor of Container class.
+
+        The following constraint attributes may be set via opts or
+        **kwargs, but are otherwise initialised to None:
+
+        - weight
+        - minimum_dose
+        - maximum_dose
+        - full_volume_dose,
+        - prescription_dose
+        - underdose_volume_fraction
+        - overdose_volume_fraction
+
+        **Parameters:**
+
+        opts: dict, default={}
+            Dictionary to be used in setting instance attributes
+            (dictionary keys) and their initial values.
+
+        `**`kwargs
+            Keyword-value pairs to be used in setting instance attributes
+            and their initial values.
+        """
+
+        for attribute in Constraint.get_weight_and_objectives():
+            setattr(self, attribute, None)
+
+        super().__init__(opts, **kwargs)
+
